// Zaglavlje simu2.h
#include <iostream>
#include <cstdlib>
#include <cmath>

// Koristimo prostor imena STL biblioteke
using namespace std;

// Prostor imena simu2
namespace simu2 {
  // Konstante
  const int max_broj_uredu  = 10;
  const int max_broj_ulisti = 50;
  // Klasa raspodela
  class raspodela {
    // Seme generatora slucajnih brojeva (GSB)
    unsigned int seme;
    // Multiplikator GSB
    unsigned int a;
    // Modulus GSB
    unsigned int m;
  public:
    // Konstruktor
    raspodela(unsigned int s = 1):seme(s) {
      // Postavljamo vrednost multiplikatora
      a = 16807;
      // Postavljamo vrednost modulusa
      m = 2147483647;
    }
    // Lehmerov multiplikativni kongruentni generator
    double operator() () {
      seme = (a*seme)%(m+1);
      return seme/(1.0*m);
    }
    // Uniformna raspodela
    double unif(double a,double b) {
      if(b>a) 
        return a+(b-a)*(*this)();
      else {
        cerr << "Gornja granica mora da bude veca od donje" << endl;
        exit(1);
      }        
    }
    // Eksponencijalna raspodela
    double expo(double m) {
      if(m>0)
        return -m*log((*this)());
      else {
        cerr << "Ocekivanje mora da bude vece od nule" << endl;
        exit(1);
      }        
    }
    // Normalna raspodela
    double norm(double m, double s) {
      double z = 0.0;
      if(s>0) {
        for(int i=0; i<12; i++)
          z += (*this)();
        return m+(z-6)*s;
      }
      else {
        cerr << "Devijacija ne sme da bude manja ili jednaka nuli" << endl;
        exit(1);
      }        
    }
  };
  // Klasa entiteta
  class entitet {
    // Omogucavamo klasi simulacija da pristupi privatnim clanovima klase entitet
    friend class simulacija; 
    // Omogucavamo klasi fifo_red da pristupi privatnim clanovima klase entitet
    friend class fifo_red;
    // Redni broj entiteta
    int id;
    // Buduci dogadjaj
    int naredni_dogadjaj;
    // Vreme nastupanja buduceg dogadjaja
    double vreme_nastupanja_dogadjaja;
  public:
    // Konstruktor
    entitet(int eid):id(eid) { }
    // Vraca id entiteta
    int vrati_id() { return id; }
  }; 
  // Klasa FIFO red cekanja
  class fifo_red {
    // Broj entiteta u redu cekanja
    int broj;
    // Polje pokazivaca na entitete
    entitet* red[max_broj_uredu];
  public:
    // Konstruktor
    fifo_red(): broj(0) {}
    // Smestamo entitet u red cekanja
    void ured(entitet* e) {
      if( broj < max_broj_uredu) {
        // Smestamo entitet na kraj reda cekanja
        red[broj]=e;
        // Uvecavamo broj entiteta za 1
        broj++;
      }
      else {
        cerr << "Previse korisnika u redu cekanja" << endl;
        exit(1);
      }
    }
    // Vadimo entitet iz reda cekanja
    entitet* izred() {
      entitet *e;
      if(broj>0) {
        // Vadimo prvi entitet iz reda cekanja
        e = red[0];
        for(int i=1;i<broj;i++)
          red[i-1] = red[i];
        // Umanjujemo broj entiteta iz reda cekanja
        broj--;
      }
      else {
        cerr << "Nema entiteta u redu cekanja" << endl;
        exit(1);
      }
      // Vraca entitet
      return e;
    }
    // Vracamo velicinu reda cekanja
    int velicina() { return broj; };
    void stampa() { 
      for(int i=0; i<broj; i++) 
        cout << red[i]->id << ",";
    }
  };
  // Klasa resursa
  class resurs {
    // Tekuci broj zauzetih mesta u resursa
    int broj_zauzetih_mesta;
    // Maksimalni broj  mesta u resursu
    int ukupni_broj_mesta;
  public:
    // Konstruktor
    resurs(int b):ukupni_broj_mesta(b),broj_zauzetih_mesta(0) {}
    // Zauzima mesto u resursu
    void zauzmi() { 
      if(broj_zauzetih_mesta<ukupni_broj_mesta)
        // Uvecavamo broj zauzetih mesta
        broj_zauzetih_mesta++;
      else {
        cerr << "Sva mesta u resursu su zauzeta" << endl;
        exit(1);
      }
    }
    // Oslobadja mesto u resursu
    void oslobodi() {  
      if(broj_zauzetih_mesta>0)
        // Umanjujemo broj zauzetih mesta
        broj_zauzetih_mesta--;
      else {
        cerr << "Nema entiteta u resursu" << endl;
        exit(1);
      }        
    }
    // Raspolozivost resursa (ima makar jedno slobodno mesto u resursu)
    bool raspoloziv() { return (broj_zauzetih_mesta<ukupni_broj_mesta); }
  };
  
  // Klasa simulacija
  class simulacija {
    // Brojac entiteta
    int eid;
    // Terminacioni brojac
    int tb;
    // Broj entiteta u LBD listi
    int broj;
    // Vreme simulacije
    double vreme_simulacije;
    // Polje entiteta u LBD listi
    entitet* lista_nastupanja_entiteta[max_broj_ulisti];
    // Algoritam brzog sortiranja
    void quicksort(entitet* x[],int l,int d) {
      int i = l, j = d;
      entitet* t;
      // Izbor pivota 
      entitet* p = x[(l+d)/2];
      // Rasporedjivanje
      while(i<=j) {
        // Pomeramo i dok ne naidjemo na element veci ili jednak pivotu
        while(x[i]->vreme_nastupanja_dogadjaja < p->vreme_nastupanja_dogadjaja)
          i++;
        // Pomeramo j dok ne naidjemo na element manji ili jednak pivotu
        while(x[j]->vreme_nastupanja_dogadjaja > p->vreme_nastupanja_dogadjaja)
          j--;
        // Ukoliko je i<=j
        if(i<=j) {
          // Razmena elemenata (swap)
          t = x[i];
          x[i] = x[j];
          x[j] = t;
          // Pomeramo  i i j na naredni, odnosno, prethodni element
          i++; j--;
        }
      };
      // Rekurzivni pozivi algoritma sortiranja nad levim i desnim podpoljem
      if(l<j) quicksort(x,l,j);
      if(i<d) quicksort(x,i,d);
    }
    // Sortiranje LBD - Sortiranje izborom
    void sortiraj_listu_nastupanja() {
      quicksort(lista_nastupanja_entiteta,0,broj-1);
    }
  public:
    // Konstruktor
    simulacija():eid(1),tb(0),broj(0),vreme_simulacije(0.0) {}
    // Destruktor
    ~simulacija() {
      // Uklanjamo preostale entitete iz LBD na kraju simulacije.
      for(int i=0; i<broj; i++)
        delete lista_nastupanja_entiteta[i];
    }
    // Pravljenje entiteta
    entitet* napravi_entitet() {
      // Pravimo novi entitet
      return new entitet(eid++);
    }
    // Unistavanje entiteta
    void unisti_entitet(entitet *e, int b) {
      // Unistavamo entitet
      delete e;
      // Umanjujemo terminacioni brojac
      tb -= b;
    }
    // Rasporedjivanje dogadjaja
    void rasporedi(entitet* e,int dog,double vreme) {
      if( broj < max_broj_ulisti) {
        // Postavljamo dogadjaj
        e->naredni_dogadjaj = dog;
        // Postavljamo vreme nastupanja dogadjaja
        e->vreme_nastupanja_dogadjaja = vreme;  
        // Smestamo entitet u listu
        lista_nastupanja_entiteta[broj] = e;
        broj++;
        // Sortiramo LBD
        sortiraj_listu_nastupanja();
      }
      else {
        cerr << "Previse entiteta u listi dogadjaja" << endl;
        exit(1);
      }        
    }
    // Izvrsavanje simulacije
    void izvrsi(int b) {
      entitet * tekuci;
      // Postavljamo terminacioni brojac
      tb = b;
      // Izvrsavamo simulaciju. Simulacija se zavrsava ukoliko
      // nema entiteta u LBD ili ukoliko je terminacioni brojac 0.
      do {
        // Vadimo prvi entitet iz liste. Taj entitet postaje tekuci entitet.
        tekuci = lista_nastupanja_entiteta[0];
        for(int i=1;i<broj;i++)
          lista_nastupanja_entiteta[i-1] = lista_nastupanja_entiteta[i];
        broj--;
        // Faza 1: Azurirarmo vreme simulacije
        vreme_simulacije = tekuci->vreme_nastupanja_dogadjaja;
        // Faza 2: Izvrsavamo dogadjaj
        izvrsavanje_dogadjaja(tekuci->naredni_dogadjaj,tekuci);
      } while(broj && tb>0);
    }
    // Izvrsava dogadjaj
    void izvrsavanje_dogadjaja(int dog, entitet* e);
    // Vraca vreme simulacije
    double vrati_vreme_simulacije() { return vreme_simulacije; }
  };
}
