// Datototeka simu2.cpp
#include "simu2.h"
#include <iostream>

// Makro naredbe
#define ID(e) (e)->vrati_id()
#define VREME simu.vrati_vreme_simulacije()
#define VRTR(vreme) simu.vrati_vreme_simulacije()+(vreme)

// Ukljucujemo prostore imena simu2 i std
using namespace simu2;
using namespace std;

// Redni broj dogadjaja DolazakKlijentaUUniverzalniSalter
const int DOLAZAK_UNI_KLIJENTA = 0;
// Redni broj dogadjaja OdlazakKlijentaIzUniverzalnogSaltera
const int ODLAZAK_UNI_KLIJENTA = 1;
// Redni broj dogadjaja DolazakKlijentaUPaketskiSalter
const int DOLAZAK_PAK_KLIJENTA = 2;
// Redni broj dogadjaja OdlazakKlijentaIzPaketskogSaltera
const int ODLAZAK_PAK_KLIJENTA = 3;
// Redni broj dogadjaja KrajSimulacije
const int KRAJ_SIMULACIJE      = 4;

// Potpis funkcije bezuslovnog dogadjaja DolazakKlijentaUUniverzalniSalter
void dolazak_uni_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja OdlazakKlijentaIzUniverzalnogSaltera
void odlazak_uni_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja DolazakKlijentaUPaketskiSalter
void dolazak_pak_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja OdlazakKlijentaIzPaketskogSaltera
void odlazak_pak_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja KrajSimulacije
void kraj_simulacije(entitet *e);

// Izvrsavanje dogadjaja
void simulacija::izvrsavanje_dogadjaja(int dog, entitet* e) {
  switch(dog) {
    case DOLAZAK_UNI_KLIJENTA:
      dolazak_uni_klijenta(e);
      break;
    case ODLAZAK_UNI_KLIJENTA:
      odlazak_uni_klijenta(e);
      break;
    case DOLAZAK_PAK_KLIJENTA:
      dolazak_pak_klijenta(e);
      break;
    case ODLAZAK_PAK_KLIJENTA:
      odlazak_pak_klijenta(e);
      break;
    case KRAJ_SIMULACIJE:
      kraj_simulacije(e);
  }
}
// Redovi cekanja
fifo_red red1,red2;
// Salteri
resurs uni_salt(2),pak_salt(1);
// Objekat simulacije
simulacija simu;
// Raspodele
raspodela rni(76531), rn1(11615), rn2(36517), rn3(61521), rn4(98737);

// Srednje vreme dolaska entiteta u univerzalni salter
const double SRVRDOLUNI  = 60;
// Srednja vremena opsluge entiteta u univerzalnom salteru u zavisnosti od usluge 
const double SRVROPSUNI[] = {40, 50, 15, 70};
// Srednje vreme dolaska entiteta u paketski salter
const double SRVRDOLPAK  = 200;
// Modifikator vremena dolaska entiteta u paketski salter
const double MODDOLPAK   = 80;
// Srednje vreme opsluge entiteta u paketskom salteru
const double SRVROPSPAK  = 150;
// Standardna devijacija vremena opsluge entiteta u paketskom salteru
const double STDEVOPSPAK = 60;
// Minimalno vreme opsluge entiteta u paketskom salteru
const double MINOPSPAK = 20;

// Dogadjaj dolazak klijenta na univerzalni salter
void dolazak_uni_klijenta(entitet *e) {
  entitet *fe, *ne;
  double pi,vreme_opsluge;
  cout << "ured_uni" << "," << ID(e) << "," << VREME << endl;
  // Postavi klijenta u red cekanja
  red1.ured(e);
  // Ukoliko je salter raspoloziv
  if(uni_salt.raspoloziv()) {
    // Izvadi prvog klijenta iz reda
    fe = red1.izred();
    cout << "izred_uni" << "," << ID(fe) << "," << VREME << endl;
    // Zauzmi jedno mesto u salteru
    uni_salt.zauzmi();
    cout << "usal_uni" << "," << ID(fe) << "," << VREME << endl;
    // Verovatnoca izbora usluge
    pi = rni();
    // Vreme opsluge na osnovu izabrane vrste usluge
    if(pi<0.20) 
      vreme_opsluge = rn2.expo(SRVROPSUNI[0]);
    else if(pi<0.60) 
      vreme_opsluge = rn2.expo(SRVROPSUNI[1]);
    else if(pi<0.75) 
      vreme_opsluge = rn2.expo(SRVROPSUNI[2]);
    else if(pi<1.00)
      vreme_opsluge = rn2.expo(SRVROPSUNI[3]);
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_UNI_KLIJENTA,VRTR(vreme_opsluge));
  }
  // Stvaramo narednog klijenta
  ne = simu.napravi_entitet();
  // Postavljamo vreme narednog dolaska
  simu.rasporedi(ne,DOLAZAK_UNI_KLIJENTA,VRTR(rn1.expo(SRVRDOLUNI)));
}
// Dogadjaj odlazak klijenta sa univerzalnog saltera
void odlazak_uni_klijenta(entitet *e) {
  entitet *fe;
  double pi,vreme_opsluge;
  // Vrati salter;
  uni_salt.oslobodi();
  cout << "izsal_uni" << "," << ID(e) << "," << VREME << endl;
  // Klijent odlazi iz poste - unistavamo tekuceg klijenta
  simu.unisti_entitet(e,0);
  // Ukoliko red nije prazan
  if(red1.velicina()>0) {
    // Izvadi prvog klijenta iz reda
    fe = red1.izred();
    cout << "izred_uni" << "," << ID(fe) << "," << VREME << endl;
    // Zauzmi jedno mesto u salteru
    uni_salt.zauzmi();
    cout << "usal_uni" << "," << ID(fe) << "," << VREME << endl;
    // Verovatnoca izbora usluge
    pi = rni();
    // Vreme opsluge na osnovu izabrane vrste usluge
    if(pi<0.20) 
      vreme_opsluge = rn2.expo(SRVROPSUNI[0]);
    else if(pi<0.60) 
      vreme_opsluge = rn2.expo(SRVROPSUNI[1]);
    else if(pi<0.75) 
      vreme_opsluge = rn2.expo(SRVROPSUNI[2]);
    else if(pi<1.00)
      vreme_opsluge = rn2.expo(SRVROPSUNI[3]);
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_UNI_KLIJENTA,VRTR(vreme_opsluge));
  }
}
// Dogadjaj dolazak klijenta na paketski salter
void dolazak_pak_klijenta(entitet *e) {
  entitet *fe, *ne;
  double vreme_opsluge;
  cout << "ured_pak" << "," << ID(e) << "," << VREME << endl;
  // Postavi klijenta u red cekanja
  red2.ured(e);
  // Ukoliko je salter raspoloziv
  if(pak_salt.raspoloziv()) {
    // Izvadi prvog klijenta iz reda
    fe = red2.izred();
    cout << "izred_pak" << "," << ID(fe) << "," << VREME << endl;
    // Zauzmi jedno mesto u salteru
    pak_salt.zauzmi();
    cout << "usal_pak" << "," << ID(fe) << "," << VREME << endl;
    do {
      vreme_opsluge = rn4.norm(SRVROPSPAK,STDEVOPSPAK);
    } while(vreme_opsluge<MINOPSPAK);
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_PAK_KLIJENTA,VRTR(vreme_opsluge));
  }
  // Stvaramo narednog klijenta
  ne = simu.napravi_entitet();
  // Postavljamo vreme narednog dolaska
  simu.rasporedi(ne,DOLAZAK_PAK_KLIJENTA,
        VRTR(rn3.unif(SRVRDOLPAK-MODDOLPAK,SRVRDOLPAK+MODDOLPAK)));
}
// Dogadjaj odlazak klijenta sa paketskog saltera
void odlazak_pak_klijenta(entitet *e) {
  entitet *fe;
  double vreme_opsluge;
  // Vrati paketski salter;
  pak_salt.oslobodi();
  cout << "izsal_pak" << "," << ID(e) << "," << VREME << endl;
  // Klijent odlazi iz poste - unistavamo tekuceg klijenta
  simu.unisti_entitet(e,0);
  // Ukoliko red ispred paketsko saltera nije prazan
  if(red2.velicina()>0) {
    // Izvadi prvog klijenta iz reda
    fe = red2.izred();
    cout << "izred_pak" << "," << ID(fe) << "," << VREME << endl;
    // Zauzmi jedno mesto u paketskom salteru
    pak_salt.zauzmi();
    cout << "usal_pak" << "," << ID(fe) << "," << VREME << endl;
    do {
      vreme_opsluge = rn4.norm(SRVROPSPAK,STDEVOPSPAK);
    } while(vreme_opsluge<MINOPSPAK);
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_PAK_KLIJENTA,VRTR(vreme_opsluge));
  }
}
// Dogadjaj kraj simulacije
void kraj_simulacije(entitet* e) {
  simu.unisti_entitet(e,1);
  // Stampamo vreme simulacije
  cout << "vsim" << "," << -1 << "," << VREME << endl;
}
int main() {
  // Stvaramo prvog klijenta
  entitet *e1 = simu.napravi_entitet();
  // Postavljamo vreme dolaska prvog klijenta
  simu.rasporedi(e1,DOLAZAK_UNI_KLIJENTA,rn1.expo(SRVRDOLUNI));
  // Stvaramo prvog klijenta
  entitet *e2 = simu.napravi_entitet();
  // Postavljamo vreme dolaska prvog klijenta
  simu.rasporedi(e2,DOLAZAK_PAK_KLIJENTA,
                rn3.unif(SRVRDOLPAK-MODDOLPAK,SRVRDOLPAK+MODDOLPAK));
  // Stvaramo entitet za kraj simulacije
  entitet *e3 = simu.napravi_entitet();
  // Postavljamo vreme zavrsetka simulacije
  simu.rasporedi(e3,KRAJ_SIMULACIJE,43200);
  // Izvrsavamo simulaciju
  simu.izvrsi(1);
}
