#include <iostream>
#include <fstream>
#include <string>
using namespace std;

const int max_id_num = 20;

class resurs_stat {
  protected:
    // Ukupni, tekuci i maksimalni broj entiteta u resursu
    int broj, tek_broj, max_broj;
    // Ukupno vreme za koje je resur bio zauzet
    double ukupno_vreme;
    // Vreme za koje se entitet zadrzava u resursu
    double povrsina_cekanja;
    // Poslednji trenutak promene stanja entiteta u resurs
    double vreme_promene;
    // Ukupno vreme simulacije
    double vreme_sim;
    // Polje rednih brojeva entiteta koji se nalaze u resursu
    int id[max_id_num];
    // Vremenski trenutak promene stanja resursa od strane entiteta 
    double vreme[max_id_num];
    // Broj entiteta u polju id i polju vreme
    int broj_id;
  public:
    resurs_stat(): broj(0),tek_broj(0),max_broj(0),
                   ukupno_vreme(0),povrsina_cekanja(0),
                   vreme_promene(0), broj_id(0) {}
    void ulaz(int idv,double vremev) {
      if(idv!=-1) {
        // Uvecamo broj entiteta koji su usli resurs
        broj++;
        //
        povrsina_cekanja += tek_broj*(vremev-vreme_promene);
        // Pamtimo trenutak poslednje promene broja zauzetih mesta
        vreme_promene = vremev;
        //  Uvecavamo tekuci broj zauzetih mesta u salteru
        tek_broj++;
        // Ukoliko je tekuci broj zauzetih mesta veci od maksimalnog
        // tekuci broj postaje maksimalni broj
        if(tek_broj>max_broj)
          max_broj = tek_broj;
        // Pamtimo redni broj entiteta u resursu i trenutak ulaska entiteta u resurs
        dodaj_id(idv,vremev);
      }
      else 
        povrsina_cekanja += tek_broj*(vremev-vreme_promene);
    }
    void izlaz(int idv,double vremev) {
      // Nalazimo poziciju entiteta u resursu
      int n = nadji_id(idv);
      // Ukupno vreme koje su entiteti proveli u salteru
      ukupno_vreme += vremev-vreme[n];
      //
      povrsina_cekanja += tek_broj*(vremev-vreme_promene);
      // Pamtimo vreme ulaska entiteta u resurs
      vreme_promene = vremev;
      // Uklanjamo redni broj entiteta sa pozicije n
      ukloni_id(n);
      //  Umanjujemo tekuci broj entiteta u resursu
      tek_broj--;
    }
  protected:
    // Dodajemo redni broj entiteta u polje id
    void dodaj_id(int novi_id,double novo_vreme) {
      if(broj_id<max_id_num) {
        id[broj_id] = novi_id;
        vreme[broj_id] = novo_vreme;
        broj_id++;
      }
      else {
        cerr << "Greska dodaj id" << endl;
        exit(1);
      }
    }
    // Nalazimo redni broj entiteta u polju id
    int nadji_id(int trazi_id) {
      int i;
      for(i=0; i<broj_id; i++) 
        if(id[i]==trazi_id)
          break;
      return i;
    }
    // Uklanjamo redni broj i tekuce vreme entiteta iz polja id i vreme sa pozicije pos
    void ukloni_id(int pos) {
      for(int i=pos+1; i<broj_id; i++) {
        id[i-1] = id[i];
        vreme[i-1] = vreme[i]; 
      }
      broj_id--;
    }
  public:
    void vreme_simulacije(double vs) { vreme_sim = vs; }
    inline int ukupno_uslih_entiteta() { return broj; }
    inline int tekuci_broj_entiteta() { return tek_broj; }
    inline int maksimalni_broj_entiteta() { return max_broj; }
    inline double srednje_vreme() {
      return ukupno_vreme/(broj-tek_broj);
    }
    inline double srednji_broj_entiteta() {
      return povrsina_cekanja/vreme_sim;
    }
    inline double iskoriscenost(int broj_kanala) {
      return srednji_broj_entiteta()/broj_kanala;
    }
};
// Klasa statistika reda cekanja
class fifo_red_stat: public resurs_stat {
  public:
    // Broj klijenata koji nisu cekali
    int broj_bezcekanja;
  public:
    fifo_red_stat():resurs_stat(),broj_bezcekanja(0) {}
    void izlaz(int idv,double vremev) {
      // Ukoliko je vreme koje je entitet proveo u redu 0
      if(vremev-vreme_promene==0.0) 
        // Uvecavamo broj entiteta koji nisu cekali u redu
        broj_bezcekanja++;    
      resurs_stat::izlaz(idv,vremev);
    }
  public:
    inline int broj_entiteta_bez_zadrzavanja() { return broj_bezcekanja; }
    inline double procenat_entiteta_bez_zadrzavanja() {
      return (100.0*broj_bezcekanja)/broj;
    }
    inline double srednji_broj_entiteta_bez_zadrzavanja() {
      return ukupno_vreme/(broj-tek_broj-broj_bezcekanja);
    }
};
void print_statistike_reda_cekanja(fifo_red_stat& red,const char* opis) {
  // Statistike reda cekanja
  cout << "STATISTIKE REDA CEKANJA - " << opis << endl;
  cout << "Broj entiteta koji je usao u red cekanja: " << red.ukupno_uslih_entiteta() << endl;
  cout << "Preostali broj entiteta u redu cekanja: " << red.tekuci_broj_entiteta() << endl;
  cout << "Maksimalni broj entiteta u redu: " << red.maksimalni_broj_entiteta() << endl;
  cout << "Srednje vreme cekanja u redu: " << red.srednje_vreme() << endl;
  cout << "Srednji broj entiteta u redu: " << red.srednji_broj_entiteta() << endl;
  cout << "Broj entiteta koji je prosao kroz red bez zadrzavanja: " << red.broj_entiteta_bez_zadrzavanja() << endl;
  cout << "Procenat ulaza bez zadrzavanja: " << red.procenat_entiteta_bez_zadrzavanja() << endl;
  cout << "Srednje vreme cekanja u redu (iskljuc. ent. koji se nisu zadrzali): " << red.srednji_broj_entiteta_bez_zadrzavanja() << endl;
  cout << endl;
}
void print_statistike_resursa(resurs_stat& salt,int n,const char* opis) {
  // Statistike resursa
  cout << "STATISTIKE RESURSA - " << opis << endl;
  cout << "Broj entiteta koji je usao u resurs: " << salt.ukupno_uslih_entiteta() << endl;
  cout << "Preostali broj entiteta u resursu: " << salt.tekuci_broj_entiteta() << endl;
  cout << "Maksimalni broj zauzetih kanala opsluge: " << salt.maksimalni_broj_entiteta() << endl;
  cout << "Srednje vreme opsluge: " << salt.srednje_vreme() << endl;
  cout << "Srednji broj zauzetih kanala: " << salt.srednji_broj_entiteta() << endl;
  cout << "Iskoriscenost: " << salt.iskoriscenost(n) << endl;
  cout << endl;
}
int main() {
  int pos,posn,idv;
  string str,tip,id,vreme;
  char line[1024];
  resurs_stat salt1, salt2;
  fifo_red_stat red1, red2;
  double vremev, vreme_sim;
  // Ucitavamo izvestaj 
  ifstream in("simu3.txt",ifstream::in);
  // Ucitavamo podatke iz izvestaja sve dok ne dodjemo do kraja datoteke
  while(in.good()) {
    // Ucitavamo liniju iz izvestaja
    in.getline(line,1024);
    // Dodlejujemo liniju stringu
    str = line;
    // Izdvajamo tip obavestenja
    posn = str.find_first_of(',');
    tip = str.substr(0,posn);
    pos = posn;
    // Izdvajamo vrednost rednog broja entiteta
    posn = str.find_first_of(',',pos+1);
    id = str.substr(pos+1,posn-pos-1);
    pos = posn;
    // Izdvajamo vremenski trenutak
    vreme = str.substr(pos+1);
    // Prevodimo id i vremenski trenutak u broj
    idv = atoi(id.c_str());
    vremev = atof(vreme.c_str());
    // Proveravamo tip obavestenja
    if(!tip.compare("ured_uni"))
      red1.ulaz(idv,vremev);
    else if(!tip.compare("izred_uni"))
      red1.izlaz(idv,vremev);
    else if(!tip.compare("ured_pak"))
      red2.ulaz(idv,vremev);
    else if(!tip.compare("izred_pak"))
      red2.izlaz(idv,vremev);
    else if(!tip.compare("usal_uni"))
      salt1.ulaz(idv,vremev);
    else if(!tip.compare("izsal_uni"))
      salt1.izlaz(idv,vremev);
    else if(!tip.compare("usal_pak"))
      salt2.ulaz(idv,vremev);
    else if(!tip.compare("izsal_pak"))
      salt2.izlaz(idv,vremev);
    else if(!tip.compare("vsim")) {
      //  Pamtimo vreme simulacije
      vreme_sim = vremev;
      // Postavljamo vreme simulacije
      red1.vreme_simulacije(vremev);
      red2.vreme_simulacije(vremev);
      salt1.vreme_simulacije(vremev);
      salt2.vreme_simulacije(vremev);
      //
      red1.ulaz(idv,vremev);
      red2.ulaz(idv,vremev);
      salt1.ulaz(idv,vremev);
      salt2.ulaz(idv,vremev);
    }
  }
  // Zatvaramo datoteku
  in.close();
  // Podesavamo preciznost izlaznih rezulata
  cout.precision(3);
  cout.setf(ios_base::fixed);
  // ISPISUJEMO STATISTIKE
  // Ispisujemo vreme simulacije
  cout << "Vreme simulacije: " << vreme_sim << endl;
  cout << endl;
  // Statistike reda cekanja
  print_statistike_reda_cekanja(red1,"UNIVERZALNI");
  print_statistike_reda_cekanja(red2,"PAKETSKI");
  // Statistike resursa
  print_statistike_resursa(salt1,2,"UNIVERZALNI");
  print_statistike_resursa(salt2,1,"PAKETSKI");
  return 0;
}
