// Datototeka simu4.cpp
#include "simul.h"
#include <iostream>
#include <iomanip>
#include <string>

// Makro naredbe
#define ID(e) (e)->vrati_id()
#define VREME simu.vrati_vreme_simulacije()
#define VRTR(vreme) simu.vrati_vreme_simulacije()+(vreme)

// Ukljucujemo prostore imena simul i std
using namespace simul;
using namespace std;

// Redni broj dogadjaja DolazakKlijentaUUniverzalniSalter
const int DOLAZAK_UNI_KLIJENTA   = 0;
// Redni broj dogadjaja OdlazakKlijentaIzUniverzalnogSaltera1
const int ODLAZAK_UNI_KLIJENTA_1 = 1;
// Redni broj dogadjaja OdlazakKlijentaIzUniverzalnogSaltera2
const int ODLAZAK_UNI_KLIJENTA_2 = 2;
// Redni broj dogadjaja DolazakKlijentaUPaketskiSalter
const int DOLAZAK_PAK_KLIJENTA   = 3;
// Redni broj dogadjaja OdlazakKlijentaIzPaketskogSaltera
const int ODLAZAK_PAK_KLIJENTA   = 4;
// Redni broj dogadjaja KrajSimulacije
const int KRAJ_SIMULACIJE        = 5;

// Potpis funkcije bezuslovnog dogadjaja DolazakKlijentaUUniverzalniSalter
void dolazak_uni_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja OdlazakKlijentaIzUniverzalnogSaltera1
void odlazak_uni_klijenta_1(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja OdlazakKlijentaIzUniverzalnogSaltera2
void odlazak_uni_klijenta_2(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja DolazakKlijentaUPaketskiSalter
void dolazak_pak_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja OdlazakKlijentaIzPaketskogSaltera
void odlazak_pak_klijenta(entitet *e);
// Potpis funkcije bezuslovnog dogadjaja KrajSimulacije
void kraj_simulacije(entitet *e);

// Izvrsavanje dogadjaja
void simulacija::izvrsavanje_dogadjaja(int dog, entitet* e) {
  switch(dog) {
    case DOLAZAK_UNI_KLIJENTA:
      dolazak_uni_klijenta(e);
      break;
    case ODLAZAK_UNI_KLIJENTA_1:
      odlazak_uni_klijenta_1(e);
      break;
    case ODLAZAK_UNI_KLIJENTA_2:
      odlazak_uni_klijenta_2(e);
      break;
    case DOLAZAK_PAK_KLIJENTA:
      dolazak_pak_klijenta(e);
      break;
    case ODLAZAK_PAK_KLIJENTA:
      odlazak_pak_klijenta(e);
      break;
    case KRAJ_SIMULACIJE:
      kraj_simulacije(e);
  }
}
// Objekat simulacije
simulacija simu;
// Redovi cekanja
red_cekanja ured1(&simu),ured2(&simu),pred(&simu);
// Salteri
resurs uni_salt1(&simu,1),uni_salt2(&simu,1),pak_salt(&simu,1);
// Histogrami
histogram histo1(50,50,20), histo2(1,2,10);
// Raspodele
raspodela rni(76531), rn1(11615), rn2(36517), rn3(61521), rn4(98737), rn5(12341);

// Vremenski interval dolaska entiteta u univerzalni salter
const double VIDOLUNI[]   = {18000, 32400, 39600};
// Srednje vreme dolaska entiteta u univerzalni salter
const double SRVRDOLUNI[] = {50.0, 35.0, 25.0};
// Verovatnoca izbora srednjeg vremena opsluge entiteta u univerzalnom salteru u zavisnosti od usluge 
const double PIOPSUNI[]   = {0.20, 0.60, 0.75, 1.0};
// Srednja vremena opsluge entiteta u univerzalnom salteru u zavisnosti od usluge 
const double SRVROPSUNI[] = {40, 50, 15, 70};
// Srednje vreme dolaska entiteta u paketski salter
const double SRVRDOLPAK   = 240;
// Srednje vreme opsluge entiteta u paketskom salteru
const double SRVROPSPAK   = 150;
// Standardna devijacija vremena opsluge entiteta u paketskom salteru
const double STDEVOPSPAK  = 60;
// Minimalno vreme opsluge entiteta u paketskom salteru
const double MINOPSPAK    = 20;

// Dogadjaj dolazak klijenta na univerzalni salter
void dolazak_uni_klijenta(entitet *e) {
  entitet *fe, *ne;
  int u_salter;
  double pi,vreme_opsluge;
  // Ukoliko je salter raspoloziv
  if(uni_salt1.raspoloziv())
    u_salter = 1;
  else if(uni_salt2.raspoloziv())
    u_salter = 2;
  else if(ured1.velicina()<=ured2.velicina())
    u_salter = 1;
  else
    u_salter = 2;
  if(u_salter == 1) {
    // U parametar jedan upisujemo vremenski trenutak ulaska u red
    (*e)[0] = VREME;
    // Postavi klijenta u red cekanja
    ured1.ured(e);
    // Ukoliko red nije prazan
    if (uni_salt1.raspoloziv()) {
      // Izvadi prvog klijenta iz reda
      fe = ured1.izred();
      // Tabeliramo vreme boravka u redu
      histo1(VREME-(*fe)[0]);
      // Zauzmi jedno mesto u salteru
      uni_salt1.zauzmi(fe);
      // Vreme opsluge
      vreme_opsluge = rn2.expo(funkcija::diskretna(rni(),PIOPSUNI,SRVROPSUNI,4));
      // Rasporedi klijenta za kraj opsluge 
      simu.rasporedi(fe,ODLAZAK_UNI_KLIJENTA_1,VRTR(vreme_opsluge));
    }
  }
  else {
    // U parametar jedan upisujemo vremenski trenutak ulaska u red
    (*e)[0] = VREME;    
    // Postavi klijenta u red cekanja
    ured2.ured(e);
    // Ukoliko red nije prazan
    if (uni_salt2.raspoloziv()) {
      // Izvadi prvog klijenta iz reda
      fe = ured2.izred();
      // Tabeliramo vreme boravka u redu
      histo1(VREME-(*fe)[0]);     
      // Zauzmi jedno mesto u salteru
      uni_salt2.zauzmi(fe);
      // Vreme opsluge
      vreme_opsluge = rn2.expo(funkcija::diskretna(rni(),PIOPSUNI,SRVROPSUNI,4));
      // Rasporedi klijenta za kraj opsluge 
      simu.rasporedi(fe,ODLAZAK_UNI_KLIJENTA_2,VRTR(vreme_opsluge));
    }
  }
  // Stvaramo narednog klijenta
  ne = simu.napravi_entitet();
  // Postavljamo vreme narednog dolaska
  simu.rasporedi(ne,DOLAZAK_UNI_KLIJENTA,VRTR(rn1.expo(funkcija::diskretna(VREME,VIDOLUNI,SRVRDOLUNI,3))));
}
// Dogadjaj odlazak klijenta sa univerzalnog saltera 1
void odlazak_uni_klijenta_1(entitet *e) {
  entitet *fe;
  double pi,vreme_opsluge;
  // Vrati salter;
  uni_salt1.oslobodi(e);
  // Klijent odlazi iz poste - unistavamo tekuceg klijenta
  simu.unisti_entitet(e,0);
  // Ukoliko red nije prazan
  if(ured1.velicina()>0) {
    // Izvadi prvog klijenta iz reda
    fe = ured1.izred();
    // Tabeliramo vreme boravka u redu
    histo1(VREME-(*fe)[0]);
    // Zauzmi jedno mesto u salteru
    uni_salt1.zauzmi(fe);
    // Vreme opsluge
    vreme_opsluge = rn2.expo(funkcija::diskretna(rni(),PIOPSUNI,SRVROPSUNI,4));
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_UNI_KLIJENTA_1,VRTR(vreme_opsluge));
  }
}
// Dogadjaj odlazak klijenta sa univerzalnog saltera 2
void odlazak_uni_klijenta_2(entitet *e) {
  entitet *fe;
  double pi,vreme_opsluge;
  // Vrati salter;
  uni_salt2.oslobodi(e);
  // Klijent odlazi iz poste - unistavamo tekuceg klijenta
  simu.unisti_entitet(e,0);
  // Ukoliko red nije prazan
  if(ured2.velicina()>0) {
    // Izvadi prvog klijenta iz reda
    fe = ured2.izred();
    // Tabeliramo vreme boravka u redu
    histo1(VREME-(*fe)[0]);
    // Zauzmi jedno mesto u salteru
    uni_salt2.zauzmi(fe);
    // Vreme opsluge
    vreme_opsluge = rn2.expo(funkcija::diskretna(rni(),PIOPSUNI,SRVROPSUNI,4));
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_UNI_KLIJENTA_2,VRTR(vreme_opsluge));
  }
}
// Dogadjaj dolazak klijenta na paketski salter
void dolazak_pak_klijenta(entitet *e) {
  entitet *fe, *ne;
  double vreme_opsluge;
  if(VREME>=3600 && VREME<=36000) {  
    // Postavi klijenta u red cekanja
    pred.ured(e);
    // Ukoliko je salter raspoloziv
    if(pak_salt.raspoloziv()) {
      // Tabeliramo duzinu reda cekanja
      histo2(pred.velicina());
      // Izvadi prvog klijenta iz reda
      fe = pred.izred();
      // Zauzmi jedno mesto u salteru
      pak_salt.zauzmi(fe);
      do {
        vreme_opsluge = rn5.norm(SRVROPSPAK,STDEVOPSPAK);
      } while(vreme_opsluge<MINOPSPAK);
      // Rasporedi klijenta za kraj opsluge 
      simu.rasporedi(fe,ODLAZAK_PAK_KLIJENTA,VRTR(vreme_opsluge));
    }
  }
  else {
    simu.unisti_entitet(e,0);
  }
  // Stvaramo narednog klijenta
  ne = simu.napravi_entitet();
  // Postavljamo vreme narednog dolaska
  simu.rasporedi(ne,DOLAZAK_PAK_KLIJENTA,
                    VRTR(rn3.expo(SRVRDOLPAK/2)+rn4.expo(SRVRDOLPAK/2)));  
}
// Dogadjaj odlazak klijenta sa paketskog saltera
void odlazak_pak_klijenta(entitet *e) {
  entitet *fe;
  double vreme_opsluge;
  // Vrati paketski salter;
  pak_salt.oslobodi(e);
  // Klijent odlazi iz poste - unistavamo tekuceg klijenta
  simu.unisti_entitet(e,0);
  // Ukoliko red ispred paketsko saltera nije prazan
  if(pred.velicina()>0) {
    // Tabeliramo duzinu reda cekanja
    histo2(pred.velicina());
    // Izvadi prvog klijenta iz reda
    fe = pred.izred();
    // Zauzmi jedno mesto u paketskom salteru
    pak_salt.zauzmi(fe);
    do {
      vreme_opsluge = rn5.norm(SRVROPSPAK,STDEVOPSPAK);
    } while(vreme_opsluge<MINOPSPAK);
    // Rasporedi klijenta za kraj opsluge 
    simu.rasporedi(fe,ODLAZAK_PAK_KLIJENTA,VRTR(vreme_opsluge));
  }
}
// Dogadjaj kraj simulacije
void kraj_simulacije(entitet* e) {
  simu.unisti_entitet(e,1);
  // Stvaramo naredni entitet
  entitet* ne = simu.napravi_entitet();
  simu.rasporedi(ne,KRAJ_SIMULACIJE,VRTR(3600.0));
}
void ocisti() {
  ured1.ocisti();
  ured2.ocisti();
  pred.ocisti();
  uni_salt1.ocisti();
  uni_salt2.ocisti();
  pak_salt.ocisti();
  histo1.resetuj();
  histo2.resetuj();
}
void stampaj_statistike_resursa(const string& ime,const resurs& res) {
	statistike stat = res.vrati_statistike();
	// Statistike resursa
	cout << "STATISTIKE RESURSA - " << ime << endl;
	cout << "Broj entiteta koji je usao u resurs: " << stat.vrati_broj() << endl;
	cout << "Preostali broj entiteta u resursu: " << stat.vrati_tekuci_broj() << endl;
	cout << "Maksimalni broj zauzetih kanala opsluge: " << stat.vrati_maksimalni_broj() << endl;
	cout << "Srednje vreme opsluge: " << stat.vrati_srednje_vreme() << endl;
	cout << "Srednji broj zauzetih kanala: " << stat.vrati_srednji_broj(VREME) << endl;
	cout << "Iskoriscenost: " << stat.vrati_iskoriscenost(VREME, res.vrati_broj_mesta()) << endl;
	cout << endl;
}
void stampaj_statistike_reda_cekanja(const string& ime,const red_cekanja& red) {
	statistike stat = red.vrati_statistike();
	// Statistike reda cekanja
	cout << "STATISTIKE REDA CEKANJA - " << ime << endl;
	cout << "Broj entiteta koji je usao u red cekanja: " << stat.vrati_broj() << endl;
	cout << "Preostali broj entiteta u redu cekanja: " << stat.vrati_tekuci_broj() << endl;
	cout << "Maksimalni broj entiteta u redu: " << stat.vrati_maksimalni_broj() << endl;
	cout << "Srednje vreme cekanja u redu: " << stat.vrati_srednje_vreme() << endl;
	cout << "Srednji broj entiteta u redu: " << stat.vrati_srednji_broj(VREME) << endl;
	cout << "Broj entiteta koji je prosao kroz red bez zadrzavanja: " << stat.vrati_broj_bez_cekanja() << endl;
	cout << "Procenat ulaza bez zadrzavanja: " << stat.vrati_ucesce_bez_cekanja() << endl;
	cout << "Srednje vreme cekanja u redu (iskljuc. ent. koji se nisu zadrzali): " << stat.vrati_srednje_vreme_bez_cekanja() << endl;
	cout << endl;
}
void stampaj_statistike_histograma(const string& ime,histogram& histo) {
	// Statistike histograma
	cout << "HISTOGRAM - " << ime << endl;
	cout << "Broj ulaza u histogram: " << histo.vrati_broj_ulaza_u_histogram() << endl;
	if (histo.vrati_frekvenciju_intervala(histo.vrati_broj_intervala() - 1) > 0){
		cout << "Prosecna vrednost overflowa: " << histo.vrati_prosecnu_vrednost_overflowa() << endl;
		cout << "Frekvencija overflowa: " << histo.vrati_frekvenciju_intervala(histo.vrati_broj_intervala() - 1) << endl;
		cout << "Verovatnoca overflowa: " << histo.vrati_verovatnocu_intervala(histo.vrati_broj_intervala() - 1) << endl;
	}
	cout << "Srednja vrednost histograma: " << histo.vrati_srednju_vrednost_histograma() << endl;
	cout << "Standardna devijacija histograma: " << histo.vrati_standardnu_devijaciju_histograma() << endl;
	cout << "Suma argumenata histograma: " << histo.vrati_sumu_argumenata_histograma() << endl;
	cout << setw(10) << "int   " << setw(10) << "frek  " << setw(10) << "vero  " << setw(10) << "umno  " << setw(10) << "dev   " << endl;
	for (int i = 0; i < histo.vrati_broj_intervala() - 1; i++){
		cout << setw(10) << histo.vrati_gornju_granicu_intervala(i) << " "
			<< setw(10) << histo.vrati_frekvenciju_intervala(i) << " "
			<< setw(10) << histo.vrati_verovatnocu_intervala(i) << " "
			<< setw(10) << histo.vrati_umnozak_od_srednje_vrednosti_intervala(i) << " "
			<< setw(10) << histo.vrati_devijaciju_od_srednje_vrednosti_intervala(i) << endl;
	}
	cout << endl;
}
void finisiraj_statistike() {
  ured1.finisiraj();
  ured2.finisiraj();
  pred.finisiraj();
  uni_salt1.finisiraj();
  uni_salt2.finisiraj();
  pak_salt.finisiraj();
}
void stampaj_statistike() {
  stampaj_statistike_resursa("uni_salt1",uni_salt1);
  stampaj_statistike_resursa("uni_salt2",uni_salt2);
  stampaj_statistike_resursa("pak_salt" ,pak_salt);
  
  stampaj_statistike_reda_cekanja("ured1",ured1);
  stampaj_statistike_reda_cekanja("ured2",ured2);
  stampaj_statistike_reda_cekanja("pred",pred);
  
  stampaj_statistike_histograma("histo1",histo1);
  stampaj_statistike_histograma("histo2",histo2);  
}
void izvrsi_dan() {
  // Stvaramo prvog klijenta
  entitet *e1 = simu.napravi_entitet();
  // Postavljamo vreme dolaska prvog klijenta
  simu.rasporedi(e1,DOLAZAK_UNI_KLIJENTA,rn1.expo(SRVRDOLUNI[0]));
  // Stvaramo prvog klijenta
  entitet *e2 = simu.napravi_entitet();
  // Postavljamo vreme dolaska prvog klijenta
  simu.rasporedi(e2,DOLAZAK_PAK_KLIJENTA,rn3.expo(SRVRDOLPAK/2)+rn4.expo(SRVRDOLPAK/2));
  // Stvaramo entitet za kraj simulacije
  entitet *e3 = simu.napravi_entitet();
  // Postavljamo vreme zavrsetka simulacije
  simu.rasporedi(e3,KRAJ_SIMULACIJE,3600.0);
  // Izvrsavamo simulaciju
  simu.izvrsi(11);
  // Finisiramo statistike
  finisiraj_statistike();
  // Stampamo statistike
  stampaj_statistike();  
  // Cistimo statistike
  ocisti();
  // Brisemo preostale entitete iz LBD  
  simu.ocisti();
}
int main() {
  cout << "Prvi dan" << endl;
  // Izvrsavamo prvi radni dan
  izvrsi_dan();
  cout << "Drugi dan" << endl;
  // Izvrsavamo drugi radni dan
  izvrsi_dan();
  return 0;
}
