#include <iostream>
#include <fstream>
#include <cmath>

using namespace std;

// Klasa tacka u ravni
class tacka {
public:
  // Difoltni konstruktor
  tacka() { x=0; y=0; }
  // Konstruktor
  tacka(int xx,int yy) {
    x = xx;
    y = yy; 
  }
public:
  // Svojstva
  void postavi_x(const int xx) { x = xx; }
  void postavi_y(const int yy) { y = yy; }
  
  int vrati_x() const { return x; }
  int vrati_y() const { return y; }  
private:
  int x, y;
};

// Graficki objekat u ravni
class objekat {
public:
  // Difoltni konstruktor
  objekat() {}
  // Konstruktor
  objekat(fstream* id):idat(id) {}
public:
  // Metode
  void crtaj() {};
public:
  // Svojstva
  void postavi_idat(fstream* id) { idat = id; }
protected:
  fstream *idat;
};

// Klasa linija u ravni
class linija: public objekat {
public:
  // Difoltni konstruktor
  linija():objekat() {}
  // Konstruktor
  linija(fstream* id,tacka& tt1,tacka& tt2); 
public:
  // Metode
  // Crtanje - preopterecenje metode crtaj iz klase objekat
  void crtaj();
private:
  // Vraca duzinu linije
  double duzina();
public:
  // Svojstva
  void postavi_t1(const tacka& tt1); 
  void postavi_t2(const tacka& tt1);
private:
  tacka t1, t2;
};

// Konstruktor
linija::linija(fstream* id,tacka& tt1, tacka& tt2):objekat(id) {
  // Inicijalizujemo vrednosti
  t1 = tt1;
  t2 = tt2;
}
// Crtanje
void linija::crtaj() {
  (*idat) << "<line x1='" << t1.vrati_x() << "' y1='" << t1.vrati_y() << "' x2='" << t2.vrati_x() << "' y2='" << t2.vrati_y() << "' stroke='blue'/>" << endl;
  (*idat) << "<text x='" << t1.vrati_x()+(t2.vrati_x()-t1.vrati_x())/2 << "' y='" << t1.vrati_y()+(t2.vrati_y()-t1.vrati_y())/2 << "' fill='orange' text-anchor='middle'>" << duzina() << "</text>" << endl;
}
// Duzina linije
double linija::duzina() {
  return sqrt(pow(t2.vrati_x()-t1.vrati_x(),2)+pow(t2.vrati_y()-t1.vrati_y(),2));
}
void linija::postavi_t1(const tacka& tt1) {
  t1 = tt1;
}
void linija::postavi_t2(const tacka& tt2) {
  t2 = tt2;
}

// Klasa pravougaonika u ravni
class pravoug: public objekat {
public:
  // Difoltni konstruktor
  pravoug():objekat() {}
  // Konstruktor
  pravoug(fstream* id,tacka& tt0,int ss,int vv): objekat(id),t0(tt0),s(ss),v(vv) {}
public:
  // Metode
  // Crtanje
  void crtaj() {
    (*idat) << "<rect x = '" << t0.vrati_x() << "' y = '" << t0.vrati_y() << "' width='" << s << "' height='" << v << "' fill='red' stroke='black'/>" << endl;
    (*idat) << "<text x='" << t0.vrati_x()+s/2 << "' y='" << t0.vrati_y()+v/2 << "' fill='white' text-anchor='middle'>" << povrsina() << "</text>" <<endl;
  }
private:
  // Vraca duzinu linije
  double povrsina() {
    return s*v;
  }
public:
  // Svojstva
  void postavi_t0(const tacka& tt0) { t0 = tt0; }
  void postavi_s(const int ss) { s = ss; }
  void postavi_v(const int vv) { v = vv; }
private:
  // Pocetna tacka
  tacka t0;
  // Duzina sirina
  int s, v;
};

class kvadrat:public pravoug {
public:
  kvadrat():pravoug() {}
  kvadrat(fstream* id,tacka& tt0,int ss):pravoug(id,tt0,ss,ss) {}
public:
  // Svojstva
  //void postavi_s(const int ss) { s = ss; v = ss; }   // U slucaju da je pravoug::s, pravoug::v protected
  void postavi_s(const int ss) {  // U slucaju da je pravoug::s, pravoug::v private
    pravoug::postavi_s(ss);
    pravoug::postavi_v(ss);
  }   
};

int main() {
  // Otvaramo izlaznu datoteku
  fstream id("izlaz.svg",fstream::out);

  // Otvaramo SVG datoteku
  id << "<svg xmlns='http://www.w3.org/2000/svg' version='1.1'>" << endl;

  tacka t1, t2;
  t1.postavi_x(50); t1.postavi_y(50);
  t2.postavi_x(500); t2.postavi_y(50);
  
  // Pozivamo konstruktor
  linija lin1(&id,t1,t2);
  // Iscrtavamo liniju
  lin1.crtaj();

  tacka t3(100,100);
  
  // Pozivamo konstruktor
  pravoug p(&id,t3,200,300);
  p.crtaj();
  
  tacka t4(400,400);
  
  // Pozivamo konstruktor
  kvadrat* k = new kvadrat;
  k->postavi_t0(t4);
  k->postavi_s(200);
  k->postavi_idat(&id);
  k->crtaj();

  // Zatvaramo SVG datoteku
  id << "</svg>" << endl;

  // Oslobadamo memoriju
  delete k;
  
  // Zatvaramo datoteku
  id.close();
  
  return 0;
};
