// Polimorfizmi
// Apstraktne klase i virtuelne funkcije
#include <iostream>
#include <fstream>
#include <cmath>

using namespace std;

// Klasa tacka u ravni
class tacka {
public:
  // Difoltni konstruktor
  tacka() { x=0; y=0; }
  // Konstruktor
  tacka(int xx,int yy) {
    x = xx;
    y = yy; 
  }
public:
  // Svojstva
  void postavi_x(const int xx) { x = xx; }
  void postavi_y(const int yy) { y = yy; }
  
  int vrati_x() const { return x; }
  int vrati_y() const { return y; }  
private:
  int x, y;
};

// Graficki objekat u ravni
// Apstraktna klasa
class objekat {
public:
  // Difoltni konstruktor
  objekat() {}
  // Konstruktor
  objekat(fstream* id):idat(id) {}
public:
  // Metode
  virtual void crtaj() = 0;
public:
  // Svojstva
  void postavi_idat(fstream* id) { idat = id; }
protected:
  fstream *idat;
};

// Klasa linija u ravni
class linija: public objekat {
public:
  // Difoltni konstruktor
  linija():objekat() {}
  // Konstruktor
  linija(fstream* id,tacka& tt1,tacka& tt2); 
public:
  // Metode
  // Crtanje - preopterecenje metode crtaj iz klase objekat
  void crtaj();
private:
  // Vraca duzinu linije
  double duzina();
public:
  // Svojstva
  void postavi_t1(const tacka& tt1); 
  void postavi_t2(const tacka& tt1);
private:
  tacka t1, t2;
};

// Konstruktor
linija::linija(fstream* id,tacka& tt1, tacka& tt2):objekat(id) {
  // Inicijalizujemo vrednosti
  t1 = tt1;
  t2 = tt2;
}
// Crtanje
void linija::crtaj() {
  (*idat) << "<line x1='" << t1.vrati_x() << "' y1='" << t1.vrati_y() << "' x2='" << t2.vrati_x() << "' y2='" << t2.vrati_y() << "' stroke='blue'/>" << endl;
  (*idat) << "<text x='" << t1.vrati_x()+(t2.vrati_x()-t1.vrati_x())/2 << "' y='" << t1.vrati_y()+(t2.vrati_y()-t1.vrati_y())/2 << "' fill='orange' text-anchor='middle'>" << duzina() << "</text>" << endl;
}
// Duzina linije
double linija::duzina() {
  return sqrt(pow(t2.vrati_x()-t1.vrati_x(),2)+pow(t2.vrati_y()-t1.vrati_y(),2));
}
void linija::postavi_t1(const tacka& tt1) {
  t1 = tt1;
}
void linija::postavi_t2(const tacka& tt2) {
  t2 = tt2;
}

// Klasa pravougaonika u ravni
class pravoug: public objekat {
public:
  // Difoltni konstruktor
  pravoug():objekat() {}
  // Konstruktor
  pravoug(fstream* id,tacka& tt0,int ss,int vv): objekat(id),t0(tt0),s(ss),v(vv) {}
public:
  // Metode
  // Crtanje
  void crtaj() {
    (*idat) << "<rect x = '" << t0.vrati_x() << "' y = '" << t0.vrati_y() << "' width='" << s << "' height='" << v << "' fill='red' stroke='black'/>" << endl;
    (*idat) << "<text x='" << t0.vrati_x()+s/2 << "' y='" << t0.vrati_y()+v/2 << "' fill='white' text-anchor='middle'>" << povrsina() << "</text>" <<endl;
  }
private:
  // Vraca duzinu linije
  double povrsina() {
    return s*v;
  }
public:
  // Svojstva
  void postavi_t0(const tacka& tt0) { t0 = tt0; }
  void postavi_s(const int ss) { s = ss; }
  void postavi_v(const int vv) { v = vv; }
private:
  // Pocetna tacka
  tacka t0;
  // Duzina sirina
  int s, v;
};

class kvadrat:public pravoug {
public:
  kvadrat():pravoug() {}
  kvadrat(fstream* id,tacka& tt0,int ss):pravoug(id,tt0,ss,ss) {}
public:
  // Svojstva
  //void postavi_s(const int ss) { s = ss; v = ss; }   // U slucaju da je pravoug::s, pravoug::v protected
  void postavi_s(const int ss) {  // U slucaju da je pravoug::s, pravoug::v private
    pravoug::postavi_s(ss);
    pravoug::postavi_v(ss);
  }   
};

class svg {
public:
  // Difoltni konstruktor
  svg();
  // Konstruktor
  svg(const char* ime_dat);
  // Destruktor
  ~svg();
public:
  // Metoda crtaj
  void crtaj();
  // Metoda dodaj graficki objekat
  void dodaj(objekat* obj);
public:
  int vrati_broj() const { return broj; }
private:
  // Polje pokazivaca na objekte koji se iscrtavaju
  objekat **objekti;
  int broj;
  fstream *idat;
};


// Difoltni konstruktor
svg::svg():broj(0) {
  // Formiramo polje objekata koje cemo iscrtavti
  objekti = new objekat*[50];
  // Otvaramo izlaznu datoteku
  idat = new fstream("izlaz.svg",fstream::out);
  // Otvaramo SVG datoteku
  (*idat) << "<svg xmlns='http://www.w3.org/2000/svg' version='1.1'>" << endl;
}

// Konstruktor
svg::svg(const char* ime_dat):broj(0) {
  // Formiramo polje objekata koje cemo iscrtavti
  objekti = new objekat*[50];
  // Otvaramo izlaznu datoteku
  idat = new fstream(ime_dat,fstream::out);
  // Otvaramo SVG datoteku
  (*idat) << "<svg xmlns='http://www.w3.org/2000/svg' version='1.1'>" << endl;
};

// Destruktor
svg::~svg() {
  // Zatvaramo SVG datoteku
  (*idat) << "</svg>" << endl;
  // Zatvaramo datoteku
  idat->close();
  // Uklanjamo objekat datotetke
  delete idat;
  // Uklanjamo polje iz memorije
  delete [] objekti;
}

void svg::dodaj(objekat* obj) {
  if(broj < 50) {
    // Dodajemo objekat
    objekti[broj] = obj;
    // Uvecavamo broj objekata
    broj++;
  }
  else 
    cerr << "Greska" << endl;
}

void svg::crtaj() {
  for(int i=0; i<broj; i++) {
    objekti[i]->postavi_idat(idat);
    objekti[i]->crtaj();
  }
}

int main() {
  svg s("z4.svg");
  
  tacka t1, t2;
  t1.postavi_x(50); t1.postavi_y(50);
  t2.postavi_x(500); t2.postavi_y(50);
  
  // Pozivamo konstruktor
  linija *lin1 = new linija;
  lin1->postavi_t1(t1);
  lin1->postavi_t2(t2);
  s.dodaj(lin1);

  tacka t3(100,100);
  // Pozivamo konstruktor
  pravoug *p = new pravoug;
  p->postavi_t0(t3);
  p->postavi_s(300);
  p->postavi_v(400);
  s.dodaj(p);
  
  tacka t4(400,400);
  // Pozivamo konstruktor
  kvadrat* k = new kvadrat;
  k->postavi_t0(t4);
  k->postavi_s(200);
  s.dodaj(k);
  
  // Iscrtavamo sve odjednom
  s.crtaj();
  
  // Ispisujemo broj objekata koji se iscrtava
  cout << "Iscrtano je " << s.vrati_broj() << " objekata." << endl;

  // Oslobadamo memoriju
  delete lin1;
  delete p;
  delete k;
  
  return 0;
};
